use serde::{Deserialize, Serialize};

#[derive(Deserialize)]
pub struct Query {
    #[serde(default = "default_page")]
    pub page: u64,
    #[serde(default = "default_per_page")]
    pub per_page: u64,
}

fn default_page() -> u64 {
    1
}

fn default_per_page() -> u64 {
    25
}

#[derive(Serialize)]
pub struct PaginatedResponse<T>
where
    T: for<'de> Serialize,
{
    pub page: u64,
    pub per_page: u64,
    pub count: usize,
    pub items: Vec<T>,
}

impl Query {
    pub fn res<T: for<'de> Serialize>(self, items: Vec<T>) -> PaginatedResponse<T> {
        PaginatedResponse {
            page: self.page,
            per_page: self.per_page,
            count: items.len(),
            items,
        }
    }
}
